//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MandelbrotMain.h"
//---------------------------------------------------------------------------
#define X_MIN 350
#define Y_MIN 350
#define X_MAX 500
#define Y_MAX 500

#define  CANVAS_OFFSET 29+68

#define TICK_HALF 5

#define TICK_INTV 20

#define MOUSE_MODE_NONE 0
#define MOUSE_MODE_DRAG 1
#define MOUSE_MODE_RESIZE 2
#define MOUSE_MODE_REDRAW 3

#define RESIZE_SIZE 10

/*---------------------------------------------------------------------------
 * MessageBox results:
 * IDABORT    Abort button was selected.
 * IDCANCEL   Cancel button was selected.
 * IDCONTINUE Continue button was selected.
 * IDIGNORE   Ignore button was selected.
 * IDNO       No button was selected.
 * IDOK       OK button was selected.
 * IDRETRY    Retry button was selected.
 * IDTRYAGAIN Try Again button was selected.
 * IDYES      Yes button was selected.
 */
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{

  float pScale;

  mbBitmap = new Graphics::TBitmap;
  mbBrush = new Graphics::TBrush;

  ImageSList = new TStringList;

  IRootNode = tvImages->Items->GetFirstNode();

  pnImagePanel->Caption = "Main Image Panel";

  ActionUpdate();
}


//---------------------------------------------------------------------------
void __fastcall TForm1::ShiftColor()
{
  for(int x = -RangeX; x < RangeX; x++)
  {
    for(int y = -RangeY; y < RangeY; y++)
    {
      if(Form1->Canvas->Pixels[x + xOS][y + yOS] != clBlack)
        Form1->Canvas->Pixels[x + xOS][y + yOS] = (TColor)(Form1->Canvas->Pixels[x + xOS][y + yOS] + 10);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::tbSaveClick(TObject *Sender)
{
  if(SaveDialog1->Execute())
  {
    pnFilename = SaveDialog1->FileName;

    mbImage->Picture->SaveToFile(pnFilename);

    StatusBar1->SimpleText = "Image saved to " + pnFilename;

  }  
}
//---------------------------------------------------------------------------


void __fastcall TForm1::tbNewClick(TObject *Sender)
{
  TSmartImage *TempSI;
  TTreeNode *TempNode;
  int TempID;

  //---Create the new smart image instance---
  TempSI = new TSmartImage(Form1, GroupBox1, IRootNode->Count);

  if(TempSI->siFileError == FE_No_Err)
  {
    //---Add the SmartImage object to the treeview---
    TempNode = tvImages->Items->AddChildObject(IRootNode, TempSI->siName, TempSI);
    TempNode->ImageIndex = 4;
    TempNode->StateIndex = 4;
    TempNode->SelectedIndex = 4;

    //--- Assign the Tree node---
    TempSI->siTNode = TempNode;

    //--- Assign the popup menu ---
    TempSI->siPMenu = pmPicNode;
    TempSI->siPMenu->Tag = IRootNode->Count - 1;

    //--- Set the index ---
    TempSI->siIndex = IRootNode->Count - 1;

    //---Set the offset position ---
    TempSI->siGBox->Left += 20 * IRootNode->Count - 1;
    TempSI->siGBox->Top += 20 * IRootNode->Count - 1;

    //--- Set the Groupbox popup menu ---
    TempSI->siGBox->PopupMenu = pmPicNode;
    TempSI->siGBox->PopupMenu->Tag = IRootNode->Count - 1;


    ActionUpdate();


    acUseCoord->Checked = TempSI->siUseCoord;
    acUseIconBox->Checked = TempSI->siUseIconBox;
    acUseColor->Checked = TempSI->siUseColor;

   }
   else
   {
    Application->MessageBoxA("File could not be opened", "File Error", MB_OK);
    delete TempSI;
   }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::RefreshSelState(void)
{
  TSmartImage* TempSI;

  for (int i = 0; i < IRootNode->Count; i++)
  {
    TempSI = (TSmartImage*)IRootNode->Item[i]->Data;
    TempSI->Select(IRootNode->Item[i]->Selected);
  }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::Deselect(void)
{
  TSmartImage* TempSI;

  for (int i = 0; i < IRootNode->Count; i++)
  {
    TempSI = (TSmartImage*)IRootNode->Item[i]->Data;
    TempSI->Select(false);
  }
}

//---------------------------------------------------------------------------

void __fastcall TForm1::tvImagesMouseDown(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
  int tempX;
  int tempY;

  tempX = Form1->Left + tvImages->Left + X;
  tempY = Form1->Top + ToolBar1->Height + tvImages->Top + Y;

  TSmartImage* TempSI = (TSmartImage*)tvImages->Selected->Data;

  if(tvImages->Selected->Level == 1 && TempSI != NULL)
  {
    StatusBar1->SimpleText = TempSI->siName;
    TempSI->siGBox->BringToFront();
    TempSI->Select(true);
    RefreshSelState();

    if(Button == mbRight)
    {
      //---Save the index of the reference image---
      pmPicNode->Tag = TempSI->siIndex;
      pmPicNode->Popup(tempX, tempY);
      acUseCoord->Checked = TempSI->siUseCoord;
      acUseIconBox->Checked = TempSI->siUseIconBox;
      acUseColor->Checked = TempSI->siUseColor;


    }

  }
  else if(tvImages->Selected->Level == 0)
  {
    if(Button == mbRight)
    {
      Deselect();
      pmPicNode->Tag = -1;
      pmTreeLevel->Popup(tempX, tempY);

    }
  }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acDrawExecute(TObject *Sender)
{
  //StatusBar1->SimpleText = Sender->ClassName();
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
  if(TempSI != NULL)
  {
    TempSI->Draw();

		ActionUpdate();
  }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acScaleUPExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->ReScale(true);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acScaleDnExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->ReScale(false);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Image1Progress(TObject *Sender, TProgressStage Stage,
      BYTE PercentDone, bool RedrawNow, const TRect &R, const AnsiString Msg)
{
  //
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acOffsetRightExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->DoOffset(true, true);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acOffsetLeftExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->DoOffset(true, false);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acOffsetUpExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->DoOffset(false, true);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acOffsetDownExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->DoOffset(false, true);
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acSaveExecute(TObject *Sender)
{
  if(SaveDialog1->Execute() && IRootNode->Count > 0)
  {
    TSmartImage* TempSI;

    TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
    if(TempSI != NULL)
    {
      TempSI->siSaveToFile(SaveDialog1->FileName);
      StatusBar1->SimpleText = "File saved to " + SaveDialog1->FileName;
    }
  }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acUseIconBoxExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
  if(TempSI != NULL)
  {
    TempSI->siUseIconBox = !TempSI->siUseIconBox;

		acUseIconBox->Checked = TempSI->siUseIconBox;
		TempSI->Draw();
		ActionUpdate();
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acUseCoordExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;

  if(TempSI != NULL)
  {
    TempSI->siUseCoord = !TempSI->siUseCoord;

		acUseCoord->Checked = TempSI->siUseCoord;
		TempSI->Draw();
		ActionUpdate();
	}
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acUseColorExecute(TObject *Sender)
{
	TSmartImage* TempSI;

	TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
	if(TempSI != NULL)
	{
		TempSI->siUseColor = !TempSI->siUseColor;

		acUseColor->Checked = TempSI->siUseColor;
		TempSI->Draw();
		ActionUpdate();
  }
}
//---------------------------------------------------------------------------


void __fastcall TForm1::acMinimizeExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
  if(TempSI != NULL)
    TempSI->MinGBox();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acMaxExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;

  if(TempSI != NULL)
    TempSI->MaxGBox();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acCloseExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  TempSI = (TSmartImage*)IRootNode->Item[pmPicNode->Tag]->Data;
  IRootNode->Item[pmPicNode->Tag]->Delete();

  if(TempSI != NULL)
    delete TempSI;

  ActionUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ActionUpdate(void)
{
  TSmartImage* TempSI;
  int ImgPresent = 0;

  //---No nodes, turn everything off except acNew---
  if(IRootNode->Count < 1)
  {
    acOpenPic->Enabled      = false;
    acClose->Enabled        = false;
    acCloseAll->Enabled     = false;
    acSave->Enabled         = false;
    acSaveAll->Enabled      = false;
    acDraw->Enabled         = false;
    acSave->Enabled         = false;
    acMax->Enabled          = false;
    acMaxAll->Enabled       = false;
    acMinimize->Enabled     = false;
    acMinimizeAll->Enabled  = false;
    acTile->Enabled         = false;
    acCollapse->Enabled     = false;
    acExpand->Enabled       = false;
    acClearImage->Enabled   = false;
    acUseCoord->Enabled     = false;
    acUseIconBox->Enabled   = false;
    acUseColor->Enabled     = false;
    acScaleUP->Enabled      = false;
    acScaleDn->Enabled      = false;
    acOffsetUp->Enabled     = false;
    acOffsetDown->Enabled   = false;
    acOffsetLeft->Enabled   = false;
    acOffsetRight->Enabled  = false;


  }
  //---We have nodes, so we can close them and draw---
  else
  {
    acOpenPic->Enabled    = true;
    acClose->Enabled      = true;
    acCloseAll->Enabled   = true;
    acDraw->Enabled       = true;
    acMax->Enabled        = true;
    acMinimize->Enabled   = true;
    acCollapse->Enabled   = true;
    acExpand->Enabled     = true;
    acUseCoord->Enabled   = true;
    acUseIconBox->Enabled = true;
    acUseColor->Enabled   = true;


    if(IRootNode->Count > 1)
    {
      acCloseAll->Enabled     = true;
      acMaxAll->Enabled       = true;
      acMinimizeAll->Enabled  = true;
      acTile->Enabled         = true;

    }
    else
    {
      acCloseAll->Enabled     = false;
      acMaxAll->Enabled       = false;
      acMinimizeAll->Enabled  = false;
      acTile->Enabled         = false;

    }


    //---Check if any of the nodes have images---
    for(int i= 0; i < IRootNode->Count; i++)
    {
      TempSI = (TSmartImage*)IRootNode->Item[i]->Data;
      if(TempSI->siImagePresent)
        ImgPresent = true;
    }



    //---Images, allow opening of images and saving---
    if(ImgPresent == 1)
    {

      acSave->Enabled         = true;
      acClearImage->Enabled   = true;
      acScaleUP->Enabled      = true;
      acScaleDn->Enabled      = true;
      acOffsetUp->Enabled     = true;
      acOffsetDown->Enabled   = true;
      acOffsetLeft->Enabled   = true;
      acOffsetRight->Enabled  = true;
    }
    else if(ImgPresent > 1)
    {
      acSaveAll->Enabled      = true;

    }
    //--No images, turn off saving---
    else if(ImgPresent < 1)
    {
      acSave->Enabled         = false;
      acSaveAll->Enabled      = false;
      acClearImage->Enabled   = false;
      acScaleUP->Enabled      = false;
      acScaleDn->Enabled      = false;
      acOffsetUp->Enabled     = false;
      acOffsetDown->Enabled   = false;
      acOffsetLeft->Enabled   = false;
      acOffsetRight->Enabled  = false;

    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::acCloseAllExecute(TObject *Sender)
{
  TSmartImage* TempSI;
  bool SaveImg = false;
  int MsgResult;

  //---Check if the any images need to be saved---
  for(int i = 0; i < IRootNode->Count; i++) {
      TempSI = (TSmartImage*)IRootNode->Item[i]->Data;
      SaveImg = TempSI->siImagePresent && !TempSI->siImageSaved;
  }

  if(SaveImg)
  {
    MsgResult = Application->MessageBoxA("You have unsaved images: Do you want to save them?",
                                         "Before you go.",
                                         MB_YESNOCANCEL);

    if(MsgResult == IDYES)
    {
      acSaveAllExecute(Sender);

      /*
      //--- Delete the Smart Image objects ---
      for(int i = 0; i < IRootNode->Count; i++)
      {
        TempSI = (TSmartImage*)IRootNode->Item[i]->Data;

        if(TempSI->siImagePresent)
        {
          SaveDialog1->FileName = TempSI->siName + ".bmp";

          if(SaveDialog1->Execute())
          {
            if(TempSI != NULL)
            {
              TempSI->siSaveToFile(SaveDialog1->FileName);
              StatusBar1->SimpleText = "File saved to " + SaveDialog1->FileName;
            }
          }
        }
      }
      */
    }
    //---No: just exit---
    else if(MsgResult == IDNO)
    {
      DelAll();
      mbCanClose = true;
    }
    //---else cancel---
    else
      mbCanClose = false;
  }
  else
  {
    DelAll();
    mbCanClose = true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DelAll(void)
{
  TSmartImage* TempSI;

  //--- Delete the Smart Image objects ---
  for(int i = 0; i < IRootNode->Count; i++)
  {
      TempSI = (TSmartImage*)IRootNode->Item[i]->Data;
      delete TempSI;
  }
  //--- Delete the nodes from the image treeview ---
  IRootNode->DeleteChildren();
}

//---------------------------------------------------------------------------


void __fastcall TForm1::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  acCloseAllExecute(Sender);

  CanClose = mbCanClose;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::acSaveAllExecute(TObject *Sender)
{
  TSmartImage* TempSI;

  //--- Delete the Smart Image objects ---
  for(int i = 0; i < IRootNode->Count; i++)
  {
    TempSI = (TSmartImage*)IRootNode->Item[i]->Data;

    if(TempSI->siImagePresent)
    {
      SaveDialog1->FileName = TempSI->siName + ".bmp";

      if(SaveDialog1->Execute())
      {
        if(TempSI != NULL)
        {
          TempSI->siSaveToFile(SaveDialog1->FileName);
          StatusBar1->SimpleText = "File saved to " + SaveDialog1->FileName;
        }
      }
    }
  }
  mbCanClose = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::tbRedClick(TObject *Sender)
{
	tbRed->Down = !tbRed->Down;	
}
//---------------------------------------------------------------------------

void __fastcall TForm1::tbGreenClick(TObject *Sender)
{
	tbGreen->Down = !tbGreen->Down;	
}
//---------------------------------------------------------------------------

void __fastcall TForm1::tbBlueClick(TObject *Sender)
{
	tbBlue->Down = !tbBlue->Down;	
}
//---------------------------------------------------------------------------

